@@ -242,7 +242,7 @@ int recv_crypto_data(ngtcp2_conn *conn,
                      ngtcp2_encryption_level encryption_level, uint64_t offset,
                      const uint8_t *data, size_t datalen, void *user_data) {
   if (!config.quiet && !config.no_quic_dump) {
-    debug::print_crypto_data(encryption_level, data, datalen);
+    debug::print_crypto_data(encryption_level, {data, datalen});
   }
 
   return ngtcp2_crypto_recv_crypto_data_cb(conn, encryption_level, offset, data,
@@ -255,12 +255,12 @@ int recv_stream_data(ngtcp2_conn *conn, uint32_t flags, int64_t stream_id,
                      uint64_t offset, const uint8_t *data, size_t datalen,
                      void *user_data, void *stream_user_data) {
   if (!config.quiet && !config.no_quic_dump) {
-    debug::print_stream_data(stream_id, data, datalen);
+    debug::print_stream_data(stream_id, {data, datalen});
   }
 
   auto c = static_cast<Client *>(user_data);
 
-  if (c->recv_stream_data(flags, stream_id, data, datalen) != 0) {
+  if (c->recv_stream_data(flags, stream_id, {data, datalen}) != 0) {
     return NGTCP2_ERR_CALLBACK_FAILURE;
   }
 
@@ -320,6 +320,9 @@ int Client::handshake_completed() {
   if (!config.quiet) {
     std::cerr << "Negotiated cipher suite is " << tls_session_.get_cipher_name()
               << std::endl;
+    if (auto group = tls_session_.get_negotiated_group(); !group.empty()) {
+      std::cerr << "Negotiated group is " << group << std::endl;
+    }
     std::cerr << "Negotiated ALPN is " << tls_session_.get_selected_alpn()
               << std::endl;
   }
@@ -331,8 +334,9 @@ int Client::handshake_completed() {
     if (datalen < 0) {
       std::cerr << "Could not encode 0-RTT transport parameters: "
                 << ngtcp2_strerror(datalen) << std::endl;
-    } else if (util::write_transport_params(config.tp_file, data.data(),
-                                            datalen) != 0) {
+    } else if (util::write_transport_params(
+                   config.tp_file,
+                   {data.data(), static_cast<size_t>(datalen)}) != 0) {
       std::cerr << "Could not write transport parameters in " << config.tp_file
                 << std::endl;
     }
@@ -468,7 +472,7 @@ void rand(uint8_t *dest, size_t destlen, const ngtcp2_rand_ctx *rand_ctx) {
 namespace {
 int get_new_connection_id(ngtcp2_conn *conn, ngtcp2_cid *cid, uint8_t *token,
                           size_t cidlen, void *user_data) {
-  if (util::generate_secure_random(cid->data, cidlen) != 0) {
+  if (util::generate_secure_random({cid->data, cidlen}) != 0) {
     return NGTCP2_ERR_CALLBACK_FAILURE;
   }
 
@@ -491,7 +495,8 @@ int do_hp_mask(uint8_t *dest, const ngtcp2_crypto_cipher *hp,
   }
 
   if (!config.quiet && config.show_secret) {
-    debug::print_hp_mask(dest, NGTCP2_HP_MASKLEN, sample, NGTCP2_HP_SAMPLELEN);
+    debug::print_hp_mask({dest, NGTCP2_HP_MASKLEN},
+                         {sample, NGTCP2_HP_SAMPLELEN});
   }
 
   return 0;
@@ -596,7 +601,7 @@ int recv_new_token(ngtcp2_conn *conn, const uint8_t *token, size_t tokenlen,
     return 0;
   }
 
-  util::write_token(config.token_file, token, tokenlen);
+  util::write_token(config.token_file, {token, tokenlen});
 
   return 0;
 }
@@ -701,14 +706,14 @@ int Client::init(int fd, const Address &local_addr, const Address &remote_addr,
     scid = config.scid;
   } else {
     scid.datalen = 17;
-    if (util::generate_secure_random(scid.data, scid.datalen) != 0) {
+    if (util::generate_secure_random({scid.data, scid.datalen}) != 0) {
       std::cerr << "Could not generate source connection ID" << std::endl;
       return -1;
     }
   }
   if (config.dcid.datalen == 0) {
     dcid.datalen = 18;
-    if (util::generate_secure_random(dcid.data, dcid.datalen) != 0) {
+    if (util::generate_secure_random({dcid.data, dcid.datalen}) != 0) {
       std::cerr << "Could not generate destination connection ID" << std::endl;
       return -1;
     }
@@ -726,7 +731,7 @@ int Client::init(int fd, const Address &local_addr, const Address &remote_addr,
     } else {
       path = std::string{config.qlog_dir};
       path += '/';
-      path += util::format_hex(scid.data, scid.datalen);
+      path += util::format_hex({scid.data, scid.datalen});
       path += ".sqlog";
     }
     qlog_ = fopen(path.c_str(), "w");
@@ -858,12 +863,8 @@ int Client::init(int fd, const Address &local_addr, const Address &remote_addr,
 }
 
 int Client::feed_data(const Endpoint &ep, const sockaddr *sa, socklen_t salen,
-                      const ngtcp2_pkt_info *pi, uint8_t *data,
-                      size_t datalen) {
-  if (datalen == 0) {
-    return 0;
-  }
-
+                      const ngtcp2_pkt_info *pi,
+                      std::span<const uint8_t> data) {
   auto path = ngtcp2_path{
       {
           const_cast<sockaddr *>(&ep.addr.su.sa),
@@ -875,7 +876,7 @@ int Client::feed_data(const Endpoint &ep, const sockaddr *sa, socklen_t salen,
       },
       const_cast<Endpoint *>(&ep),
   };
-  if (auto rv = ngtcp2_conn_read_pkt(conn_, &path, pi, data, datalen,
+  if (auto rv = ngtcp2_conn_read_pkt(conn_, &path, pi, data.data(), data.size(),
                                      util::timestamp());
       rv != 0) {
     std::cerr << "ngtcp2_conn_read_pkt: " << ngtcp2_strerror(rv) << std::endl;
@@ -908,7 +909,7 @@ int Client::on_read(const Endpoint &ep) {
   msg.msg_iov = &msg_iov;
   msg.msg_iovlen = 1;
 
-  uint8_t msg_ctrl[CMSG_SPACE(sizeof(int)) + CMSG_SPACE(sizeof(uint16_t))];
+  uint8_t msg_ctrl[CMSG_SPACE(sizeof(int)) + CMSG_SPACE(sizeof(int))];
   msg.msg_control = msg_ctrl;
 
   for (;;) {
@@ -924,16 +925,23 @@ int Client::on_read(const Endpoint &ep) {
       break;
     }
 
+    // Packets less than 22 bytes never be a valid QUIC packet.
+    if (nread < 22) {
+      ++pktcnt;
+
+      continue;
+    }
+
     pi.ecn = msghdr_get_ecn(&msg, su.storage.ss_family);
     auto gso_size = msghdr_get_udp_gro(&msg);
     if (gso_size == 0) {
       gso_size = static_cast<size_t>(nread);
     }
 
-    auto data = buf.data();
+    auto data = std::span{buf.data(), static_cast<size_t>(nread)};
 
     for (;;) {
-      auto datalen = std::min(static_cast<size_t>(nread), gso_size);
+      auto datalen = std::min(data.size(), gso_size);
 
       ++pktcnt;
 
@@ -945,20 +953,25 @@ int Client::on_read(const Endpoint &ep) {
                   << std::dec << " " << datalen << " bytes" << std::endl;
       }
 
+      // Packets less than 22 bytes never be a valid QUIC packet.
+      if (datalen < 22) {
+        break;
+      }
+
       if (debug::packet_lost(config.rx_loss_prob)) {
         if (!config.quiet) {
           std::cerr << "** Simulated incoming packet loss **" << std::endl;
         }
-      } else {
-        feed_data(ep, &su.sa, msg.msg_namelen, &pi, data, datalen);
+      } else if (feed_data(ep, &su.sa, msg.msg_namelen, &pi,
+                           {data.data(), datalen}) != 0) {
+        return -1;
       }
 
-      nread -= datalen;
-      if (nread == 0) {
+      data = data.subspan(datalen);
+
+      if (data.empty()) {
         break;
       }
-
-      data += datalen;
     }
 
     if (pktcnt >= 10) {
@@ -1119,8 +1132,8 @@ int Client::write_streams() {
 
     auto &ep = *static_cast<Endpoint *>(ps.path.user_data);
 
-    if (auto rv =
-            send_packet(ep, ps.path.remote, pi.ecn, tx_.data.data(), nwrite);
+    if (auto rv = send_packet(ep, ps.path.remote, pi.ecn,
+                              {tx_.data.data(), static_cast<size_t>(nwrite)});
         rv != NETWORK_ERR_OK) {
       if (rv != NETWORK_ERR_SEND_BLOCKED) {
         ngtcp2_ccerr_set_liberr(&last_error_, NGTCP2_ERR_INTERNAL, nullptr, 0);
@@ -1456,11 +1469,11 @@ int Client::update_key(uint8_t *rx_secret, uint8_t *tx_secret,
 
   if (!config.quiet && config.show_secret) {
     std::cerr << "application_traffic rx secret " << nkey_update_ << std::endl;
-    debug::print_secrets(rx_secret, secretlen, rx_key.data(), keylen, rx_iv,
-                         ivlen);
+    debug::print_secrets({rx_secret, secretlen}, {rx_key.data(), keylen},
+                         {rx_iv, ivlen});
     std::cerr << "application_traffic tx secret " << nkey_update_ << std::endl;
-    debug::print_secrets(tx_secret, secretlen, tx_key.data(), keylen, tx_iv,
-                         ivlen);
+    debug::print_secrets({tx_secret, secretlen}, {tx_key.data(), keylen},
+                         {tx_iv, ivlen});
   }
 
   return 0;
@@ -1486,7 +1499,7 @@ void Client::start_delay_stream_timer() {
 }
 
 int Client::send_packet(const Endpoint &ep, const ngtcp2_addr &remote_addr,
-                        unsigned int ecn, const uint8_t *data, size_t datalen) {
+                        unsigned int ecn, std::span<const uint8_t> data) {
   if (debug::packet_lost(config.tx_loss_prob)) {
     if (!config.quiet) {
       std::cerr << "** Simulated outgoing packet loss **" << std::endl;
@@ -1495,8 +1508,8 @@ int Client::send_packet(const Endpoint &ep, const ngtcp2_addr &remote_addr,
   }
 
   iovec msg_iov;
-  msg_iov.iov_base = const_cast<uint8_t *>(data);
-  msg_iov.iov_len = datalen;
+  msg_iov.iov_base = const_cast<uint8_t *>(data.data());
+  msg_iov.iov_len = data.size();
 
   msghdr msg{};
 #ifdef HAVE_LINUX_RTNETLINK_H
@@ -1549,7 +1562,7 @@ int Client::send_packet(const Endpoint &ep, const ngtcp2_addr &remote_addr,
     return NETWORK_ERR_FATAL;
   }
 
-  assert(static_cast<size_t>(nwrite) == datalen);
+  assert(static_cast<size_t>(nwrite) == data.size());
 
   if (!config.quiet) {
     std::cerr << "Sent packet: local="
@@ -1600,7 +1613,7 @@ int Client::send_blocked_packet() {
   };
 
   auto rv = send_packet(*tx_.blocked.endpoint, remote_addr, tx_.blocked.ecn,
-                        tx_.data.data(), tx_.blocked.datalen);
+                        {tx_.data.data(), tx_.blocked.datalen});
   if (rv != 0) {
     if (rv == NETWORK_ERR_SEND_BLOCKED) {
       assert(wev_.fd == tx_.blocked.endpoint->fd);
@@ -1647,7 +1660,8 @@ int Client::handle_error() {
   }
 
   return send_packet(*static_cast<Endpoint *>(ps.path.user_data),
-                     ps.path.remote, pi.ecn, buf.data(), nwrite);
+                     ps.path.remote, pi.ecn,
+                     {buf.data(), static_cast<size_t>(nwrite)});
 }
 
 int Client::on_stream_close(int64_t stream_id, uint64_t app_error_code) {
@@ -1795,9 +1809,10 @@ int Client::submit_http_request(const Stream *stream) {
 }
 
 int Client::recv_stream_data(uint32_t flags, int64_t stream_id,
-                             const uint8_t *data, size_t datalen) {
-  auto nconsumed = nghttp3_conn_read_stream(
-      httpconn_, stream_id, data, datalen, flags & NGTCP2_STREAM_DATA_FLAG_FIN);
+                             std::span<const uint8_t> data) {
+  auto nconsumed =
+      nghttp3_conn_read_stream(httpconn_, stream_id, data.data(), data.size(),
+                               flags & NGTCP2_STREAM_DATA_FLAG_FIN);
   if (nconsumed < 0) {
     std::cerr << "nghttp3_conn_read_stream: " << nghttp3_strerror(nconsumed)
               << std::endl;
@@ -1868,11 +1883,11 @@ namespace {
 int http_recv_data(nghttp3_conn *conn, int64_t stream_id, const uint8_t *data,
                    size_t datalen, void *user_data, void *stream_user_data) {
   if (!config.quiet && !config.no_http_dump) {
-    debug::print_http_data(stream_id, data, datalen);
+    debug::print_http_data(stream_id, {data, datalen});
   }
   auto c = static_cast<Client *>(user_data);
   c->http_consume(stream_id, datalen);
-  c->http_write_data(stream_id, data, datalen);
+  c->http_write_data(stream_id, {data, datalen});
   return 0;
 }
 } // namespace
@@ -1892,8 +1907,7 @@ void Client::http_consume(int64_t stream_id, size_t nconsumed) {
   ngtcp2_conn_extend_max_offset(conn_, nconsumed);
 }
 
-void Client::http_write_data(int64_t stream_id, const uint8_t *data,
-                             size_t datalen) {
+void Client::http_write_data(int64_t stream_id, std::span<const uint8_t> data) {
   auto it = streams_.find(stream_id);
   if (it == std::end(streams_)) {
     return;
@@ -1907,7 +1921,7 @@ void Client::http_write_data(int64_t stream_id, const uint8_t *data,
 
   ssize_t nwrite;
   do {
-    nwrite = write(stream->fd, data, datalen);
+    nwrite = write(stream->fd, data.data(), data.size());
   } while (nwrite == -1 && errno == EINTR);
 }
 
@@ -3155,8 +3169,7 @@ int main(int argc, char **argv) {
     }
   }
 
-  if (util::generate_secret(config.static_secret.data(),
-                            config.static_secret.size()) != 0) {
+  if (util::generate_secret(config.static_secret) != 0) {
     std::cerr << "Unable to generate static secret" << std::endl;
     exit(EXIT_FAILURE);
   }
